@@ -3,17 +3,23 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 // Activate checking.
 #ifndef DUNE_FMatrix_WITH_CHECKING
 #define DUNE_FMatrix_WITH_CHECKING
 #endif
-#include <dune/common/fmatrix.hh>
-#include <dune/common/classname.hh>
-#include <iostream>
+
 #include <algorithm>
-#include <vector>
 #include <cassert>
 #include <complex>
+#include <iostream>
+#include <vector>
+
+#include <dune/common/classname.hh>
+#include <dune/common/fmatrix.hh>
+#include <dune/common/rangeutilities.hh>
+#include <dune/common/simd.hh>
+#include <dune/common/vc.hh>
 
 #include "checkmatrixinterface.hh"
 
@@ -271,16 +277,84 @@ void test_matrix()
   // print vector
   std::cout << f << std::endl;
 
-
+  A[0][0] += 5; // Make matrix non-zero
   {
-    FieldMatrix<K,n,m> A2 = A;
-    A2 *= 2;
+    // Test that operator= and opeator-= work before we can test anything else
+    using FM = FieldMatrix<K,n,m>;
+    FM A0 = A;
+    {
+      if (A0.infinity_norm() < 1e-12)
+        DUNE_THROW(FMatrixError, "Assignment had no effect!");
+    }
+    A0 -= A;
+    {
+      if (A0.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Operator-= had no effect!");
+    }
+    FM A1 = A;         // A1 == A
+    FM A2 = (A1 *= 2); // A1 == A2 == 2*A
+    {
+      FM tmp = A1; tmp -= A;
+      if (tmp.infinity_norm() < 1e-12)
+        DUNE_THROW(FMatrixError,"Operator*= had no effect!");
+    }
+    {
+      FM tmp = A2; tmp -= A1;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Return value of Operator*= incorrect!");
+    }
+    FM A3 = (A2 *= 3); // A2 == A3 == 6*A
+    FM A4 = (A2 /= 2); // A2 == A4 == 3*A;
+    FM A5 = A;
+    A5 *= 3;           // A5       == 3*A
+    {
+      FM tmp = A2; tmp -= A5;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Operator/= had no effect!");
+    }
+    {
+      FM tmp = A4; tmp -= A5;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Return value of Operator/= incorrect!");
+    }
 
-    FieldMatrix<K,n,m> B = A;
-    B += A;
-    B -= A2;
-    if (std::abs(B.infinity_norm()) > 1e-12)
-      DUNE_THROW(FMatrixError,"Operator +=/-= test failed!");
+    FM A6 = A;
+    FM A7 = (A6 += A); // A6 == A7 == 2*A
+    {
+      FM tmp = A1; tmp -= A6;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Operator+= had no effect!");
+    }
+    {
+      FM tmp = A1; tmp -= A7;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Return value of Operator+= incorrect!");
+    }
+
+    FM A8 = A2;        // A8 == A2 == 3*A
+    FM A9 = (A8 -= A); // A9 == A8 == 2*A;
+    {
+      FM tmp = A8; tmp -= A1;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Operator-= had no effect!");
+    }
+    {
+      FM tmp = A9; tmp -= A1;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Return value of Operator-= incorrect!");
+    }
+    FM A10 = A;
+    FM A11 = A10.axpy(2, A); // A11 = 3*A;
+    {
+      FM tmp = A10; tmp -= A2;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "axpy() had no effect!");
+    }
+    {
+      FM tmp = A10; tmp -= A11;
+      if (tmp.infinity_norm() > 1e-12)
+        DUNE_THROW(FMatrixError, "Return value of axpy() incorrect!");
+    }
   }
   {
     FieldMatrix<K,n,m> A3 = A;
@@ -348,18 +422,20 @@ void test_matrix()
   }
 }
 
+template<class T>
 int test_determinant()
 {
   int ret = 0;
 
-  FieldMatrix<double, 4, 4> B;
+  FieldMatrix<T, 4, 4> B;
   B[0][0] =  3.0; B[0][1] =  0.0; B[0][2] =  1.0; B[0][3] =  0.0;
   B[1][0] = -1.0; B[1][1] =  3.0; B[1][2] =  0.0; B[1][3] =  0.0;
   B[2][0] = -3.0; B[2][1] =  0.0; B[2][2] = -1.0; B[2][3] =  2.0;
   B[3][0] =  0.0; B[3][1] = -1.0; B[3][2] =  0.0; B[3][3] =  1.0;
-  if (std::abs(B.determinant() + 2.0) > 1e-12)
+  if (any_true(std::abs(B.determinant() + 2.0) > 1e-12))
   {
-    std::cerr << "Determinant 1 test failed" << std::endl;
+    std::cerr << "Determinant 1 test failed (" << Dune::className<T>() << ")"
+              << std::endl;
     ++ret;
   }
 
@@ -367,13 +443,14 @@ int test_determinant()
   B[1][0] = -1.0; B[1][1] =  3.0; B[1][2] =  0.0; B[1][3] =  0.0;
   B[2][0] = -3.0; B[2][1] =  0.0; B[2][2] = -1.0; B[2][3] =  2.0;
   B[3][0] = -1.0; B[3][1] =  3.0; B[3][2] =  0.0; B[3][3] =  2.0;
-  if (B.determinant() != 0.0)
+  if (any_true(B.determinant() != 0.0))
   {
-    std::cerr << "Determinant 2 test failed" << std::endl;
+    std::cerr << "Determinant 2 test failed (" << Dune::className<T>() << ")"
+              << std::endl;
     ++ret;
   }
 
-  return 0;
+  return ret;
 }
 
 template<class ft>
@@ -604,6 +681,8 @@ void test_initialisation()
 int main()
 {
   try {
+    int errors = 0; // counts errors
+
     {
       double nan = std::nan("");
       test_nan(nan);
@@ -640,14 +719,20 @@ int main()
     test_ev<double>();
 #endif
     // test high level methods
-    test_determinant();
+    errors += test_determinant< double >();
+#if HAVE_VC
+    errors += test_determinant< Vc::SimdArray<double, 8> >();
+#endif
     test_invert< float, 34 >();
     test_invert< double, 34 >();
     test_invert< std::complex< long double >, 2 >();
-    return test_invert_solve();
+    errors += test_invert_solve();
+
+    return (errors > 0 ? 1 : 0); // convert error count to unix exit status
   }
   catch (Dune::Exception & e)
   {
     std::cerr << "Exception: " << e << std::endl;
+    return 1;
   }
 }
